/* Copyright (c) 2018 -- 2022
   Charles Jordan (Hokkaido University, Sapporo, Japan),
   Michael Joswig, Lars Kastner (Technische Universitaet Berlin, Germany)

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
-----------------------------------------------------------------------------
*/

#ifndef FLIPDATA_NOSYM_HH
#define FLIPDATA_NOSYM_HH

#include "FlipData_enhanced.hh"

template<typename FlipChecker, typename APCType>
class FlipDataNoSym: public FlipData_enhanced<FlipChecker, APCType,FlipDataNoSym<FlipChecker,APCType>, FlipInfo>{
   friend class FlipData_enhanced<FlipChecker,APCType,FlipDataNoSym<FlipChecker,APCType>, FlipInfo>;
   using pmFlipContainer = typename FlipData_enhanced<FlipChecker, APCType,FlipDataNoSym<FlipChecker,APCType>, FlipInfo>::pmFlipContainer;

   private:

      Int is_lex_upflip(Flip flip, const TriangNode& target, gkz_vector_type& result_gkz){
         return this->apc.is_lex_upflip(flip, result_gkz);

      }

      void insert_flip(Flip flip, gkz_vector_type flip_gkz, Int isUpflip){
         if(isUpflip == 1){
            this->upflips.insert(flip_gkz, flip);
         } else {
            this->downflips.insert(flip_gkz, flip);
         }
      }
      
      void get_target_triangulation(const Flip& flip, FlipInfo& fi) const{
         fi.target = TriangNode(0, this->source_triang, flip);
      }
      
      void optimize_source(){
         this->source_optimal = this->source_triang;
      }
      
      
      // Mutation constructor, should not be public
      FlipDataNoSym(
            const APCType& apc_in,
            TriangNode triang,
            const pmFlipContainer& uf,
            const pmFlipContainer& df,
            const std::unordered_map<Flip, FlipInfo>& af,
            const FlipChecker& fc_in
            ): FlipData_enhanced<FlipChecker, APCType, FlipDataNoSym, FlipInfo>(apc_in, triang, uf, df, af, fc_in)
         {
            optimize_source();
         }

   public:
      typedef FlipDataNoSym<typename FlipChecker::Initial, APCType> Initial;
      typedef FlipChecker Checker;

      FlipDataNoSym(
            const APCType& apc_in,
            TriangNode triang,
            const FlipChecker& fc_in
            ): FlipData_enhanced<FlipChecker, APCType, FlipDataNoSym, FlipInfo>(apc_in, triang, fc_in)
         {}

};

#endif
