/* Copyright (c) 2018 -- 2022
   Charles Jordan (Hokkaido University, Sapporo, Japan),
   Michael Joswig, Lars Kastner (Technische Universitaet Berlin, Germany)

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
-----------------------------------------------------------------------------
*/

#ifndef FLIPDATA_SYM_HH
#define FLIPDATA_SYM_HH
#include "FlipData_enhanced.hh"



template<typename FlipChecker, typename APCType>
class FlipDataSym: public FlipData_enhanced<FlipChecker, APCType,FlipDataSym<FlipChecker,APCType>, FlipInfoSym>{
   friend class FlipData_enhanced<FlipChecker,APCType,FlipDataSym<FlipChecker,APCType>, FlipInfoSym>;
   using pmFlipContainer = typename FlipData_enhanced<FlipChecker, APCType,FlipDataSym<FlipChecker,APCType>, FlipInfoSym>::pmFlipContainer;

   private:
      // methods:
      Int is_lex_upflip(Flip flip, const TriangNode& target, gkz_vector_type& result_gkz){
         Int result;
         result = this->apc.is_lex_upflip(flip, this->source_optimal, target, result_gkz);
         return result;
      }

      void insert_flip(Flip flip, gkz_vector_type flip_gkz, Int isUpflip){
         if(isUpflip == 1){
            if(!this->upflips.exists(flip_gkz)){
               this->upflips[flip_gkz] = flip;
            }
         } else if(isUpflip == -1){
            if(!this->downflips.exists(flip_gkz)){
               this->downflips[flip_gkz] = flip;
            }
         }
      }
      

      void get_target_triangulation(const Flip& flip, FlipInfoSym& fi) const{
         fi.raw_target = TriangNode(0, this->source_triang, flip);
         const std::pair<TriangNode, Symmetry>& opt(this->apc.get_sbb().get_canonical_representative_with_perm(fi.raw_target));
         fi.target = opt.first;
         fi.optimizer = opt.second;
      }

      void optimize_source(){
         this->source_optimal = this->apc.get_sbb().get_canonical_representative(this->source_triang);
      }
      
      // Mutation constructor, should not be public
      FlipDataSym(
            const APCType& apc_in,
            TriangNode triang,
            const pmFlipContainer& uf,
            const pmFlipContainer& df,
            const std::unordered_map<Flip, FlipInfoSym>& af,
            const FlipChecker& fc_in
            ): FlipData_enhanced<FlipChecker, APCType, FlipDataSym, FlipInfoSym>(apc_in, triang, uf, df, af, fc_in)
         {
            optimize_source();
         }

   public:
      typedef FlipDataSym<typename FlipChecker::Initial, APCType> Initial;
      typedef FlipChecker Checker;

      FlipDataSym(
            const APCType& apc_in,
            TriangNode triang,
            const FlipChecker& fc_in
            ): FlipData_enhanced<FlipChecker, APCType, FlipDataSym, FlipInfoSym>(apc_in, triang, fc_in)
         {}

      FlipDataSym& operator=(const FlipDataSym& fds){
         this->all_flips = fds.all_flips;
         this->source_triang = fds.source_triang;
         this->source_optimal = fds.source_optimal;
         this->upflips = fds.upflips;
         this->downflips = fds.downflips;
         return *this;
      }

      void action(const Symmetry& g) {
         // Since up- and downflips are determined via canonical
         // representatives, up- and downflips stay the same under the group
         // action.
         this->source_triang = g(this->source_triang);
         std::unordered_map<Flip, FlipInfoSym> new_all_flips;
         for(const auto& p: this->all_flips){
            new_all_flips[g(p.first)] = p.second;
         }
         pmFlipContainer newuf, newdf;
         for(const auto& p: this->upflips){
            newuf[p.first] = g(p.second);
         }
         for(const auto& p: this->downflips){
            newdf[p.first] = g(p.second);
         }
         this->upflips = newuf;
         this->downflips = newdf;
         this->all_flips = new_all_flips;
      }


};


#endif
